#![allow(dead_code)]

use intbits::Bits;
use x86_64::structures::paging::page_table;

pub struct PageMapLevel5 {
    entries: [PageMapLevel5Entry; 512],
}

pub struct PageMapLevel5Entry {
    value: u64,
}

impl PageMapLevel5Entry {
    pub fn execute_disable(&self) -> bool {
        self.value.bit(63)
    }
    pub fn present(&self) -> bool {
        self.value.bit(0)
    }
    pub fn set_present(&mut self, present: bool) {
        self.value.set_bit(0, present);
    }
    pub fn writable(&self) -> bool {
        self.value.bit(1)
    }
    pub fn set_writable(&mut self, writable: bool) {
        self.value.set_bit(1, writable);
    }
    pub fn user_accessible(&self) -> bool {
        self.value.bit(2)
    }
    pub fn set_user_accessible(&mut self, accessible: bool) {
        self.value.set_bit(2, accessible);
    }
    pub fn write_through(&self) -> bool {
        self.value.bit(3)
    }
    pub fn set_write_through(&mut self, write_through: bool) {
        self.value.set_bit(3, write_through);
    }
    pub fn cache_disable(&self) -> bool {
        self.value.bit(4)
    }
    pub fn set_cache_disable(&mut self, cache_disable: bool) {
        self.value.set_bit(4, cache_disable);
    }
    pub fn accessed(&self) -> bool {
        self.value.bit(5)
    }
    pub fn set_accessed(&mut self, accessed: bool) {
        self.value.set_bit(5, accessed);
    }
    pub fn physical_address(&self) -> usize {
        self.value.bits(12..=51) as usize
    }
    pub fn set_physical_address(&mut self, address: u64) {
        self.value &= 0xFFF0000000000FFF;
        self.value |= (address & 0xFFFFFFFFFF) << 12;
    }
    pub fn available(&self) -> usize {
        (self.value.bits(52..=62) << 4 & self.value.bits(8..=11) << 1 & self.value.bits(6..=6))
            as usize
    }
}

pub struct PageMapLevel4 {
    entries: [PageMapLevel4Entry; 512],
}

pub struct PageMapLevel4Entry {
    value: u64,
}

impl PageMapLevel4Entry {
    pub fn execute_disable(&self) -> bool {
        self.value.bit(63)
    }
    pub fn present(&self) -> bool {
        self.value.bit(0)
    }
    pub fn set_present(&mut self, present: bool) {
        self.value.set_bit(0, present);
    }
    pub fn writable(&self) -> bool {
        self.value.bit(1)
    }
    pub fn set_writable(&mut self, writable: bool) {
        self.value.set_bit(1, writable);
    }
    pub fn user_accessible(&self) -> bool {
        self.value.bit(2)
    }
    pub fn set_user_accessible(&mut self, accessible: bool) {
        self.value.set_bit(2, accessible);
    }
    pub fn write_through(&self) -> bool {
        self.value.bit(3)
    }
    pub fn set_write_through(&mut self, write_through: bool) {
        self.value.set_bit(3, write_through);
    }
    pub fn cache_disable(&self) -> bool {
        self.value.bit(4)
    }
    pub fn set_cache_disable(&mut self, cache_disable: bool) {
        self.value.set_bit(4, cache_disable);
    }
    pub fn accessed(&self) -> bool {
        self.value.bit(5)
    }
    pub fn set_accessed(&mut self, accessed: bool) {
        self.value.set_bit(5, accessed);
    }
    pub fn physical_address(&self) -> usize {
        self.value.bits(12..=51) as usize
    }
    pub fn set_physical_address(&mut self, address: u64) {
        self.value &= 0xFFF0000000000FFF;
        self.value |= (address & 0xFFFFFFFFFF) << 12;
    }
    pub fn set_pointer_table(&mut self, table: &PageDirectoryPointerTable) {
        self.set_physical_address(&raw const table as u64);
    }
    pub fn available(&self) -> usize {
        (self.value.bits(52..=62) << 4 & self.value.bits(8..=11) << 1 & self.value.bits(6..=6))
            as usize
    }
}

pub struct PageDirectoryPointerTable {
    entries: [*mut PageDirectory; 4],
}

pub struct PageDirectoryPointer {
    value: u64,
}

impl PageDirectoryPointer {
    pub fn execute_disable(&self) -> bool {
        self.value.bit(63)
    }
    pub fn present(&self) -> bool {
        self.value.bit(0)
    }
    pub fn set_present(&mut self, present: bool) {
        self.value.set_bit(0, present);
    }
    pub fn writable(&self) -> bool {
        self.value.bit(1)
    }
    pub fn set_writable(&mut self, writable: bool) {
        self.value.set_bit(1, writable);
    }
    pub fn user_accessible(&self) -> bool {
        self.value.bit(2)
    }
    pub fn set_user_accessible(&mut self, accessible: bool) {
        self.value.set_bit(2, accessible);
    }
    pub fn write_through(&self) -> bool {
        self.value.bit(3)
    }
    pub fn set_write_through(&mut self, write_through: bool) {
        self.value.set_bit(3, write_through);
    }
    pub fn cache_disable(&self) -> bool {
        self.value.bit(4)
    }
    pub fn set_cache_disable(&mut self, cache_disable: bool) {
        self.value.set_bit(4, cache_disable);
    }
    pub fn accessed(&self) -> bool {
        self.value.bit(5)
    }
    pub fn set_accessed(&mut self, accessed: bool) {
        self.value.set_bit(5, accessed);
    }
    pub fn physical_address(&self) -> *mut PageDirectory {
        self.value.bits(12..=51) as *mut PageDirectory
    }
    pub fn set_physical_address(&mut self, address: u64) {
        self.value &= 0xFFF0000000000FFF;
        self.value |= (address & 0xFFFFFFFFFF) << 12;
    }
    pub fn set_page_directory(&mut self, dir: &PageDirectory) {
        self.set_physical_address(&raw const dir as u64);
    }
    pub fn available(&self) -> usize {
        (self.value.bits(52..=62) << 4 & self.value.bits(8..=11) << 1 & self.value.bits(6..=6))
            as usize
    }
}

pub struct PageDirectory {
    entries: [PageDirectoryEntry; 512],
}

pub struct PageDirectoryEntry {
    value: u64,
}

impl PageDirectoryEntry {
    pub fn execute_disable(&self) -> bool {
        self.value.bit(63)
    }
    pub fn present(&self) -> bool {
        self.value.bit(0)
    }
    pub fn set_present(&mut self, present: bool) {
        self.value.set_bit(0, present);
    }
    pub fn writable(&self) -> bool {
        self.value.bit(1)
    }
    pub fn set_writable(&mut self, writable: bool) {
        self.value.set_bit(1, writable);
    }
    pub fn user_accessible(&self) -> bool {
        self.value.bit(2)
    }
    pub fn set_user_accessible(&mut self, accessible: bool) {
        self.value.set_bit(2, accessible);
    }
    pub fn write_through(&self) -> bool {
        self.value.bit(3)
    }
    pub fn set_write_through(&mut self, write_through: bool) {
        self.value.set_bit(3, write_through);
    }
    pub fn cache_disable(&self) -> bool {
        self.value.bit(4)
    }
    pub fn set_cache_disable(&mut self, cache_disable: bool) {
        self.value.set_bit(4, cache_disable);
    }
    pub fn accessed(&self) -> bool {
        self.value.bit(5)
    }
    pub fn set_accessed(&mut self, accessed: bool) {
        self.value.set_bit(5, accessed);
    }
    pub fn physical_address(&self) -> usize {
        self.value.bits(12..=51) as usize
    }
    pub fn set_physical_address(&mut self, address: u64) {
        self.value &= 0xFFF0000000000FFF;
        self.value |= (address & 0xFFFFFFFFFF) << 12;
    }
    pub fn available(&self) -> usize {
        (self.value.bits(52..=62) << 4 & self.value.bits(8..=11) << 1 & self.value.bits(6..=6))
            as usize
    }
}

pub use page_table::PageTable;
